@@ -1,4 +1,5 @@
 #TODO: I NEED TO ADD GETTERS AND SETTERS FOR Y AND X BECAUSE THEY NEED unicurses.touchwin(self.parent.win)
+#TODO: I NEED TO CHECK FOR WRITE/READ/EXECUTE PERMISSIONS (PREVENT EXCEPTIONS\ERRORS) 
 
 from           glob import glob
 from           time import time
@@ -126,6 +127,19 @@ class TUIFIManager:  # TODO: I need to create a TUIWindowManager class where i w
     
     #def get_TUIFIProfile_by_key(self, key): # https://stackoverflow.com/a/2974082/11465149       
         #return next((v for k, v in TUIFIProfiles.items() if (key == k) or (isinstance(k,tuple) and key in k)),FILE_profile)
+        
+    
+    def get_profile(self, file_directory):
+        if os.path.isdir(file_directory):
+            temp_profile = TUIFIProfiles.get("empty_folder")
+            for suffix in self.suffixes:
+                if not len(glob(file_directory + sep + suffix)) == 0:
+                    temp_profile = TUIFIProfiles.get('folder')
+                    break
+        else:
+            file_extension = os.path.splitext(file_directory)[1]
+            temp_profile   = TUIFIProfiles.get(file_extension.lower(),DEFAULT_PROFILE) # ..[-1] = extension  
+        return temp_profile
     
     
     def load_files(self, directory, suffixes=None, sort_by=None):  # DON'T load and then don't show :P
@@ -144,22 +158,11 @@ class TUIFIManager:  # TODO: I need to create a TUIWindowManager class where i w
             
         max_h, pCOLS, count, y, x = 0, self.visibleW, 0, PADDING_TOP, PADDING_LEFT 
         for f in glob_files:
-            is_link  = os.path.islink(f)
-            filename = f.split(sep)[-1]
-              
-            if os.path.isdir(f):
-                temp_profile = TUIFIProfiles.get("empty_folder")
-                for suffix in self.suffixes:
-                    if not len(glob(f + sep + suffix)) == 0:
-                        temp_profile = TUIFIProfiles.get('folder')
-                        break
-            else:
-                file_extension = os.path.splitext(filename)[1]
-                temp_profile = TUIFIProfiles.get(file_extension.lower(),DEFAULT_PROFILE) # ..[-1] = extension                                                    
-                
-            file_ = TUIFile(filename,y,x,temp_profile,is_link=is_link)
+            is_link      = os.path.islink(f)
+            filename     = f.split(sep)[-1]
+            file_        = TUIFile(filename, y, x, self.get_profile(f), is_link=is_link)
+            tempX        = (PADDING_LEFT + file_.profile.width + PADDING_RIGHT)
             self.files.append(file_)
-            tempX = (PADDING_LEFT + file_.profile.width + PADDING_RIGHT)
             
             if x > pCOLS-tempX + PADDING_RIGHT - self.X:
                 x  = PADDING_LEFT
@@ -339,7 +342,7 @@ class TUIFIManager:  # TODO: I need to create a TUIWindowManager class where i w
     __temp__copied_files       = [] 
     __temp_dir_of_copied_files = ''
         
-    events = { # Disable or replace Events if you want (with 0x11111111? and None for CTRL(x)? i think) 
+    events = { # Disable or replace Events if you want (with 0x11111111? and None for CTRL(x)? i think) | UPDATE: lol wtf i did, i could just used them instead of if-elif pointing to functions but ...
         'BUTTON1_DOUBLE_CLICKED': unicurses.BUTTON1_DOUBLE_CLICKED       ,  # Temp because https://github.com/wmcbrine/PDCurses/issues/130 
         'KEY_MOUSE'             : unicurses.KEY_MOUSE                    ,
         'BUTTON4_PRESSED'       : unicurses.BUTTON4_PRESSED              ,
@@ -364,19 +367,22 @@ class TUIFIManager:  # TODO: I need to create a TUIWindowManager class where i w
         'CTRL_UP'               : 'kUP5'                                 ,
         'CTRL_LEFT'             : 'kLFT5'                                ,
         'ALT_DOWN'              : 'kDN3'                                 ,
+        'KEY_F5'                : unicurses.KEY_F(5)                     ,
         'CTRL_R'                : unicurses.CTRL('R')                    ,
         'CTRL_X'                : unicurses.CTRL('X')                    ,  
         'CTRL_C'                : unicurses.CTRL('C')                    ,  
-        'CTRL_V'                : unicurses.CTRL('V')                      
+        'CTRL_V'                : unicurses.CTRL('V')                    ,  
+        'CTRL_N'                : unicurses.CTRL('N')                    ,  
+        'CTRL_W'                : unicurses.CTRL('W')                    , 
     }  
     def __delete_file(self,file):
         if isinstance(file, TUIFile):
             file = self.directory + sep + file.name
         elif not isinstance(file, str):
             raise Exception('TUIFileTypeError: file must be of type string or TUIFile.')
-        if not os.path.isdir(file):
+        if os.path.isfile(file): # checking if exists too.
             os.remove(file)
-        else:
+        elif os.path.exists(file) and not file.endswith(sep + '..'): # "and not .." whatever
             shutil.rmtree(file)    
         self.__count_selected -= 1
             
@@ -396,7 +402,7 @@ class TUIFIManager:  # TODO: I need to create a TUIWindowManager class where i w
     __is_cut = False
     def cut(self):
         """
-        Cut-copies the selected files
+        Cut-copies the selected files | Not fully implemented yet
         """
         self.__is_cut = True  # TODO: DON'T FORGET TO CHANGE TERMUX CUT WHEN NEW VERSION[...]
         self.__copy()
@@ -404,7 +410,7 @@ class TUIFIManager:  # TODO: I need to create a TUIWindowManager class where i w
          
     def copy(self, signum=None, frame=None):
         """
-        Copies the selected files (ignore signum=None, frame=None [...])
+        Copies the selected files (ignore signum=None, frame=None [...]) | Not fully implemented yet
         """ 
         self.__is_cut = False
         self.__copy()    
@@ -414,8 +420,7 @@ class TUIFIManager:  # TODO: I need to create a TUIWindowManager class where i w
         """
         this function is TEMPOTATY and will be REMOVED, 
         it will be pressent until i find a way of drawing/managing cutted files efficiently
-        """
-        
+        """   
         self.__temp_dir_of_copied_files = self.directory
         if self.__count_selected == 1 and self.__clicked_file and not self.__clicked_file.name == '..':                                        
             self.__temp__copied_files = [self.__clicked_file]                 
@@ -438,7 +443,7 @@ class TUIFIManager:  # TODO: I need to create a TUIWindowManager class where i w
                         shutil.copyfile(f_path_name,self.directory + sep + f.name, follow_symlinks=False)
                         if self.__is_cut:
                             os.remove(f_path_name) 
-                    elif not f_path_name == self.directory and os.path.isdir(f_path_name):  # Does 'directory' exist?                
+                    elif os.path.isdir(f_path_name):  # Does 'directory' exist?                
                         shutil.copytree(f_path_name,self.directory + sep + f.name, symlinks=True)
                         if self.__is_cut:
                             shutil.rmtree(f_path_name)                             
@@ -449,10 +454,10 @@ class TUIFIManager:  # TODO: I need to create a TUIWindowManager class where i w
     
     def delete(self):
         """
-        Deletes the selected file(s).
+        Deletes the selected file(s). | Not fully implemented yet
         """
-        if self.__clicked_file and not self.__clicked_file.name == '..':
-            if self.__count_selected == 1:
+        if self.__count_selected == 1 and self.__clicked_file : 
+            if not self.__clicked_file.name == '..': # checking under __delete_file too but nvm cause i have no time right now
                 self.__delete_file(self.__clicked_file)
                 temp_i = self.__index_of_clicked_file - 1
                 self.reload()
@@ -460,34 +465,113 @@ class TUIFIManager:  # TODO: I need to create a TUIWindowManager class where i w
                 self.__clicked_file = self.files[temp_i]
                 self.__pre_clicked_file = None # hmm.. sus?
                 self.select(self.__clicked_file)
-            else: # if self.__count_selected > 1:  # Why do i even > 1 very sus
+        else: # if self.__count_selected > 1:  # Why do i even > 1 very sus
+            if self.__clicked_file:
                 temp_i = self.__index_of_clicked_file - self.__count_selected 
-                for f in self.files:
-                    if f.is_selected:
-                        self.__delete_file(f)
-                        if self.__count_selected == 0:
-                            self.reload()
-                            self.__index_of_clicked_file = temp_i
-                            self.__clicked_file = self.files[temp_i]
-                            self.__pre_clicked_file = None # hmm.. sus
-                            self.select(self.__clicked_file)
-                            break
+            else:
+                temp_i = 0 # VERY SUS BUT NVM NOW
+                
+            for f in self.files:
+                if f.is_selected:
+                    self.__delete_file(f)
+                    if self.__count_selected == 0:
+                        self.reload()
+                        self.__index_of_clicked_file = temp_i
+                        self.__clicked_file = self.files[temp_i]
+                        self.__pre_clicked_file = None # hmm.. sus
+                        self.select(self.__clicked_file)
+                        break
+   
+   
+    def rename(self):
+        if self.__clicked_file: 
+            self.escape_event_consumed = True
+            self.__clicked_file.draw_name(self.pad, self.__clicked_file.name, '', 0, unicurses.A_UNDERLINE)  # Yeah ok, whatever
+            self.__temp_name        = self.__clicked_file.name
+            self.__temp_pre_name    = self.__temp_name
+                    
+    
+    escape_event_consumed          = False
+    __change_escape_event_consumed = False  # on second loop 
+    __temp_pre_name                = ''
+    __temp_name                    = ''
+    __temp_i                       = 0
+    __illegal_filename_characters  = ('<', '>', ':',  '/', '\\', '|', '?', '*')
+    def handle_rename_events(self, event):  # At this momment i don't even care about optimizing anything... just kidding, you get the point, no free time
+        if event == unicurses.KEY_LEFT:
+            if not self.__temp_i == 0: self.__temp_i -= 1
+        elif event == unicurses.KEY_RIGHT:
+            if not self.__temp_i == len(self.__temp_name): self.__temp_i += 1
+        elif event in (unicurses.KEY_BACKSPACE, 8, 127, 263):
+            if not self.__temp_i == 0:
+                self.__temp_i -= 1
+                self.__temp_name = self.__temp_name[0:self.__temp_i] + self.__temp_name[self.__temp_i+1:]
+        elif unicurses.RCCHAR(event) in self.__illegal_filename_characters:
+            return
+        elif event == 27 or event in (unicurses.KEY_ENTER,10):
+            self.__temp_i                       = 0
+            self.__change_escape_event_consumed = True
+            new_path_name                       = self.directory + sep + self.__temp_name
+            if not event == 27 and not self.__temp_name.strip() == '' and not os.path.exists(new_path_name):  
+                os.rename(self.directory + sep + self.__clicked_file.name, new_path_name)        
+                self.__clicked_file.name = self.__temp_name
+                self.__clicked_file.profile = self.get_profile(new_path_name)
+                self.resort()
+                self.scroll_to_file(self.__clicked_file, True)
+            else:
+                self.__temp_name = self.__clicked_file.name
+        else:
+            self.__temp_name = self.__temp_name[0:self.__temp_i] + unicurses.RCCHAR(event) + self.__temp_name[self.__temp_i:]
+            self.__temp_i += 1  
+        self.__clicked_file.draw_name(self.pad,self.__temp_name,self.__temp_pre_name, self.__temp_i)
+        self.__temp_pre_name = self.__temp_name
+        #if event in (unicurses.KEY_BACKSPACE, 8, 127, 263):
+        #    pass
+    
+    
+    def create_new(self,_type='folder'): # temporary implementation but nvm      
+        i, j = '', 0
+        if _type == 'folder': exists = os.path.isdir
+        else                : exists = os.path.isfile 
+        while exists(self.directory + sep + 'New ' + _type + i):  
+            i = ' (' + str(j) + ')'
+            j += 1 
+        filename = 'New ' + _type + i
+        if _type == 'folder': 
+            os.mkdir(self.directory + sep + filename)
+            _type = 'empty_folder'
+        else                : 
+            open(self.directory + sep + filename, 'w').close()
+        self.deselect()
+        self.__clicked_file = TUIFile(filename, profile=TUIFIProfiles.get(_type))
+        self.__index_of_clicked_file = 1
+        self.files.insert(1,self.__clicked_file)
+        self.resort()
+        self.scroll_to_file(self.__clicked_file,True)
+        self.rename()
+    
                         
-                        
-    def __perform_menu_selected_action(self, action):
-        if   action == 'Open'  : 
-            self.open(self.__clicked_file)
-        elif action == 'Cut'   : 
-            self.cut()
-        elif action == 'Delete': 
-            self.delete()
-        elif action == 'Copy'  : 
-            self.copy()
-        elif action == 'Paste' : 
-            self.paste()
+    def __perform_menu_selected_action(self, action):  # TODO: USE DICT INSTEAD OF IF, ELIF
+        if   action == 'Open'      : self.open  (self.__clicked_file)
+        elif action == 'Cut'       : self.cut   ()
+        elif action == 'Delete'    : self.delete()
+        elif action == 'Copy'      : self.copy  ()
+        elif action == 'Paste'     : self.paste ()
+        elif action == 'Rename'    : self.rename()
+        elif action == 'New File'  : self.create_new('file')
+        elif action == 'New Folder': self.create_new('folder')
+        elif action == 'Reload'    : self.reload()
             
                             
     def handle_events(self, event): # wtf, ok .. works acceptably :P, TODO: REMOVE rrrrepeating code but nvm for now >:( xD
+        if self.escape_event_consumed == True: # REDIRECT ALL KEYBOARD EVENTS 
+            if not self.__change_escape_event_consumed: 
+                self.handle_rename_events(event)
+                return
+            else:
+                self.__change_escape_event_consumed = False
+                self.escape_event_consumed = False
+        
         action = self.menu.handle_keyboard_events(event)  # if performed the event   
         if action: # action-εστί που λεμε και στη βυζαντινη
             self.__perform_menu_selected_action(action)         
@@ -512,12 +596,12 @@ class TUIFIManager:  # TODO: I need to create a TUIWindowManager class where i w
   
                     
                     if (self.__mouse_btn1_pressed_file == self.__clicked_file and not bstate & self.events.get('BUTTON_CTRL')) :
-                        if not ((bstate & self.events.get('BUTTON3_RELEASED')) and self.__count_selected > 1 and  self.__clicked_file.is_selected): 
+                        if not ((bstate & self.events.get('BUTTON3_RELEASED')) and self.__count_selected > 1 and self.__clicked_file and self.__clicked_file.is_selected): 
                             self.menu.delete()
                             self.deselect()
                         if (bstate & self.events.get('BUTTON3_RELEASED')):
                             self.menu.create(y,x)
-                        if self.__mouse_btn1_pressed_file and not self.__mouse_btn1_pressed_file.name == '..':
+                        if self.__mouse_btn1_pressed_file and not self.__mouse_btn1_pressed_file.name == '..' and not self.__mouse_btn1_pressed_file.is_selected :
                             self.select(self.__mouse_btn1_pressed_file )
                         if (((sumed_time < self.double_click_DELAY) and (bstate & self.events.get('BUTTON1_RELEASED'))) or bstate & self.events.get('BUTTON1_DOUBLE_CLICKED')) and self.__clicked_file: #and count == 2  :
                             self.open(self.__clicked_file)
@@ -544,20 +628,23 @@ class TUIFIManager:  # TODO: I need to create a TUIWindowManager class where i w
                             self.deselect(self.__mouse_btn1_pressed_file)
                             
                     self.__pre_clicked_file = self.__mouse_btn1_pressed_file 
-            else:
+            else:  # TERMUX 
                 if bstate & self.events.get('BUTTON1_CLICKED') or bstate & self.events.get('BUTTON1_PRESSED'): # TERMUX                
-                    clicked_file = self.get_tuifile_by_coordinates(y, x)
+                    self.__index_of_clicked_file, self.__clicked_file = self.get_tuifile_by_coordinates(y, x, return_enumerator=True) 
                     
                     if not self.is_on_termux_select_mode: # bstate & BUTTON_CTRL :
                         self.deselect()
                             
-                    if self.is_on_termux_select_mode and clicked_file and not clicked_file.name == '..' : # bstate & BUTTON_CTRL 
-                        if not clicked_file.is_selected : 
-                            self.select(clicked_file)
+                    if self.is_on_termux_select_mode and self.__clicked_file and not self.__clicked_file.name == '..' : # bstate & BUTTON_CTRL 
+                        if not self.__clicked_file.is_selected : 
+                            self.select(self.__clicked_file)
                         else:                   
-                            self.deselect(clicked_file)
-                    elif clicked_file:
-                        self.open(clicked_file)
+                            self.deselect(self.__clicked_file)
+                    elif self.__clicked_file:
+                        if self.is_on_termux_select_mode: # AUTO COPY IF IN SELECT-MODE AND CLICKED ON  '..'
+                            self.is_on_termux_select_mode = False
+                            self.copy()
+                        self.open(self.__clicked_file)
            
         elif event == unicurses.KEY_RESIZE: # maybe i will add a calculate_size() function for this code part that will call a function from the TUIWindowManager 
             new_lines, new_columns = unicurses.getmaxyx(self.parent.win)
@@ -582,7 +669,7 @@ class TUIFIManager:  # TODO: I need to create a TUIWindowManager class where i w
             self.parent.columns = new_columns
             self.resort()
             unicurses.touchwin(self.parent.win)
-            
+        
         elif event in self.events.get('KEY_BACKSPACE'):
             self.open(self.directory + sep + '..')
             
@@ -662,21 +749,14 @@ class TUIFIManager:  # TODO: I need to create a TUIWindowManager class where i w
                 self.__clicked_file = self.files[temp_i]
                 self.select(self.__clicked_file)
                 
-        elif event == self.events.get('KEY_DC'): # TODO: FIX ISSUE WHEN NAVIGATING trough link and deleting | Update i think i fixed it lol
-            self.delete()
-                    
-        elif event == self.events.get('CTRL_R'):
-            self.reload()  
-            
-        elif event == self.events.get('CTRL_C'):  # or KEY_IC ? | copy selected files
-            self.copy()
-             
-        elif event == self.events.get('CTRL_X'):  
-            self.cut() 
-            
-        elif event == self.events.get('CTRL_V'): # check if path the  same as self.directory maybe? 
-            self.paste()
-                
+        elif event == self.events.get('CTRL_R'): self.rename()
+        elif event == self.events.get('KEY_DC'): self.delete()  # TODO: FIX ISSUE WHEN NAVIGATING trough link and deleting | Update i think i fixed it lol
+        elif event == self.events.get('KEY_F5'): self.reload()  
+        elif event == self.events.get('CTRL_C'): self.copy  ()  # or KEY_IC ? | copy selected files
+        elif event == self.events.get('CTRL_X'): self.cut   ()  
+        elif event == self.events.get('CTRL_V'): self.paste ()  # check if path the  same as self.directory maybe?
+        elif event == self.events.get('CTRL_W'): self.create_new('file')
+        elif event == self.events.get('CTRL_N'): self.create_new('folder')        
         elif event == unicurses.CTRL('f'):
             pass # find
         
